﻿define(["sitecore", "/-/speak/v1/campaignmanager/dictionary.js", "/-/speak/v1/campaignmanager/langResolver.js", "/-/speak/v1/campaignmanager/antiForgeryUtils.js"], function (sitecore, dictionary, langResolver, antiForgeryUtils) {
  "use strict";
  // Constants
  var defaultItemId = "-1",
    controlsMap = {
      "{FFAEA943-DF50-4F11-94ED-4690418A7D96}": "ClassificationGroup",
      "{4C1BF83B-06E2-4B07-AD16-0E65D95E356E}": "ClassificationChannel",
      "{3C0EEC17-C74D-4921-9577-D41AEA11B6DB}": "ClassificationMarketingAsset",
      "{297AB1FA-9A3B-483D-B8F7-41E7BBE549FB}": "ClassificationFacet1",
      "{13F24114-C29B-4ACB-A39B-445CCE2ED60C}": "ClassificationFacet2",
      "{336BA862-E974-4CD6-BE4D-4BFDD78994E1}": "ClassificationFacet3",
      "{82ABE216-2119-4803-BC9B-BB69571A973B}": "ClassificationFacet4",
      "{9F517EEC-EE1A-4D36-8808-A76CF2FA46E7}": "ClassificationFacet5",
      "{60B84CA2-BB8E-4C8C-95F6-495057DAD4D4}": "ClassificationFacet6",
      "{C713F777-C1DC-45E5-A86F-F504B864E4AB}": "ClassificationFacet7"
    },
    valueControlsMap = {},
    pendingInitializationValue = {};

  // Private functions
  var populateValueControls = function (dataSource) {
    var response = dataSource.get("response");
    var data = JSON.parse(response);

    valueControlsMap[this.name] = {};

    for (var i = 0; i < data.length; i++) {
      var facet = data[i];
      var controlNameBegin = controlsMap[facet.FieldId];
      if (!controlNameBegin) {
        continue;
      }

      var label = this[controlNameBegin + "Label"];
      if (!label) {
        continue;
      }

      var value = this[controlNameBegin + "Value"];
      if (!value) {
        continue;
      }

      var name = facet.Name + ":";
      label.set("text", name);

      var values = facet.Values;
      values.unshift({ Id: defaultItemId, Name: this.StringDictionary.get(dictionary.PleaseSelect) });
      value.set("items", values);
      valueControlsMap[this.name][facet.FieldId] = value;
    }

    if (typeof pendingInitializationValue[this.name] !== "undefined") {
      this.setValue(pendingInitializationValue[this.name]);
    }

    sitecore.trigger("cm:classifications:initialized");

    trackChanges.call(this);
  },
  getControlValue = function (control) {
    if (control.componentName == "ComboBox") {
      var value = control.get("selectedItemId");
      return (value !== defaultItemId) ? value : "";
    } else {
      return "";
    }
  },
  setControlValue = function (value, control) {
    var items = control.get("items");
    var selectedItems = [];

    if (typeof items !== "undefined" && items !== null) {
      selectedItems = items.filter(function (el) {
        return el.Id === value;
      });
    }

    if (control.componentName == "ComboBox") {
      control.set("selectedItems", selectedItems);
    } else {
      var names = [];

      selectedItems.forEach(function(item) {
        names.push(item.Name);
      });

      control.set("text", names.join());
    }
  },
  trackChanges = function() {
    var valueControls = [];

    for (var key in this) {
      var control = this[key];
      if (control && control.componentName === "ComboBox") {
        valueControls.push(control);
      }
    }

    valueControls.forEach(function (valueControl) {
      valueControl.on("change:selectedItemId", controlValueChanged, this);
    });
  },
  controlValueChanged = function () {
    sitecore.trigger("cm:classifications:change:value");
  };

  var extensionObject = {
    initialized: function () {
      this.ClassificationsDataSource.on("change:hasResponse", populateValueControls, this);

      var lang = langResolver.resolve();
      var url = this.ClassificationsDataSource.get("url");
      url = url + "?sc_lang=" + lang;
      this.ClassificationsDataSource.set("url", url);

      this.ClassificationsDataSource = antiForgeryUtils.getDecoratorFor(this.ClassificationsDataSource);

      this.ClassificationsDataSource.refresh();
    },
    getValue: function () {
      var returnValue = [];
      for (var fieldId in valueControlsMap[this.name]) {
        var control = valueControlsMap[this.name][fieldId];
        var selectedValue = getControlValue(control);
        if (selectedValue !== "") {
          returnValue.push({ FieldId: fieldId, Value: selectedValue });
        }
      }

      return returnValue;
    },
    setValue: function (value) {
      if (typeof valueControlsMap[this.name] === "undefined") {
        valueControlsMap[this.name] = {};
        pendingInitializationValue[this.name] = value;
      }

      for (var i = 0; i < value.length; i++) {
        var facet = value[i];
        var valueControl = valueControlsMap[this.name][facet.FieldId];

        if (typeof valueControl !== "undefined") {
          setControlValue(facet.Value, valueControl);
        }
      }
    }
  };

  return sitecore.Definitions.App.extend(extensionObject);
});